package db.util;

public class ConvertUtils {
    public static String bytes2HexArray(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        sb.append("[");
        String tmp = null;
        for (byte b : bytes) {
            // 将每个字节与0xFF进行与运算，然后转化为10进制，然后借助于Integer再转化为16进制
            tmp = Integer.toHexString(0xFF & b);
            if (tmp.length() == 1) {
                tmp = "0" + tmp;
            }
            sb.append(tmp + " ");
        }
        sb.append("]");

        return sb.toString();
    }

    public static String bytes2HexArrayWithIndex(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        sb.append("[");
        String tmp = null;
        int index = 0;
        for (byte b : bytes) {
            // 将每个字节与0xFF进行与运算，然后转化为10进制，然后借助于Integer再转化为16进制
            tmp = Integer.toHexString(0xFF & b);
            if (tmp.length() == 1) {
                tmp = "0" + tmp;
            }
            sb.append("(" + index + ")");
            sb.append(tmp + " ");
            index++;
        }
        sb.append("]");

        return sb.toString();
    }

    public static String bytes2Hex(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        String tmp = null;
        for (byte b : bytes) {
            // 将每个字节与0xFF进行与运算，然后转化为10进制，然后借助于Integer再转化为16进制
            tmp = Integer.toHexString(0xFF & b);
            if (tmp.length() == 1) {
                tmp = "0" + tmp;
            }
            sb.append(tmp);
        }

        return sb.toString();
    }

    public static byte[] hex2Bytes(String s) {
        int len = s.length();
        byte[] b = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            // 两位一组，表示一个字节,把这样表示的16进制字符串，还原成一个字节
            b[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4) + Character
                    .digit(s.charAt(i + 1), 16));
        }
        return b;
    }

    public static String ip2String(int ip) {
//        StringBuilder builder = new StringBuilder();
//        builder.append(String.valueOf(ip >> 24) + ".");
//        builder.append(String.valueOf((ip & 0x00FFFFFF) >> 16) + ".");
//        builder.append(String.valueOf((ip & 0x0000FFFF) >> 8) + ".");
//        builder.append(String.valueOf((ip & 0x000000FF)));
//        return builder.toString();

        StringBuilder sb = new StringBuilder();
        int num = 0;
        boolean needPoint = false; // 是否需要加入'.'
        for (int i = 0; i < 4; i++) {
            if (needPoint) {
                sb.append('.');
            }
            needPoint = true;
            int offset = 8 * (3 - i);
            num = (ip >> offset) & 0xff;
            sb.append(num);
        }
        return sb.toString();
    }


    public static String transportProtocolType(byte protocol) {
        if (protocol == 17) {
            return "UDP";
        } else if (protocol == 1) {
            return "ICMP";
        } else if (protocol == 2) {
            return "IGMP";
        } else if (protocol == 6) {
            return "TCP";
        } else {
            return "UNKNOWN" + protocol;
        }
    }

    public static String ipProtocolType(byte[] ipType) {
        if (ipType == null || ipType.length != 2) return "UNKNOWN";

        if (ipType[0] == 0x08 && ipType[1] == 0x06) {
            return "ARP";
        }
        return String.format("0x%02x%02x", ipType[0], ipType[1]);
    }

    public static long ipToInt(String ipStr) {
        String[] ip = ipStr.split("\\.");
        return (Integer.parseInt(ip[0]) << 24) + (Integer.parseInt(ip[1]) << 16) + (Integer.parseInt(ip[2]) << 8) + Integer.parseInt(ip[3]);
    }

    public static void arrayCopy(byte[] srcArr, int srcPos, byte[] desArr, int desPos, int length) {

        if (srcPos < 0 || desPos < 0 || length > srcArr.length) {
            return;// 不合理
        }
        for (int i = 0; i < length; i++) {
            desArr[desPos + i] = srcArr[srcPos + i];
        }
    }

    /**
     * @param n
     * @Title: intTohex
     * @Description: int型转换成16进制
     * @return: String
     */
    public static String long2hex(long n) {
        StringBuffer s = new StringBuffer();
        String a;
        char[] b = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
        while (n != 0) {
            s = s.append(b[(int) (n % 16)]);
            n = n / 16;
        }
        a = s.reverse().toString();
        if ("".equals(a)) {
            a = "00";
        }
        if (a.length() == 1) {
            a = "0" + a;
        }
        return a;
    }

    public static long macBytes2Long(byte[] bytes) {
        long value = 0;
        for (int i = 0; i < 6; i++) {
            value <<= 8;
            value |= (bytes[i] & 0xff);
        }
        return value;
    }

    public static byte[] long2macBytes(long mac) {
        byte[] buffer = new byte[6];
        for (int i = 0; i < 6; i++) {
            int offset = 6 * 8 - (i + 1) * 8;
            buffer[i] = (byte) ((mac >> offset) & 0xff);
        }
        return buffer;
    }

    public static String longMac2String(long mac) {
        return String.format("%12s", Long.toHexString(mac)).replace(" ", "0");
    }

    public static int bytes2Int(byte[] bytes) {
        int value = 0;
        for (int i = 0; i < 4; i++) {
            value <<= 8;
            value |= (bytes[i] & 0xff);
        }
        return value;
    }

    public static byte[] int2Bytes(int value) {
        byte[] buffer = new byte[4];
        for (int i = 0; i < 4; i++) {
            int offset = 4 * 8 - (i + 1) * 8;
            buffer[i] = (byte) ((value >> offset) & 0xff);
        }
        return buffer;
    }

    public static short bytes2Short(byte[] bytes) {
        short value = 0;
        for (int i = 0; i < 2; i++) {
            value <<= 8;
            value |= (bytes[i] & 0xff);
        }
        return value;
    }

    public static byte[] short2Bytes(int value) {
        byte[] buffer = new byte[2];
        for (int i = 0; i < 2; i++) {
            int offset = 2 * 8 - (i + 1) * 8;
            buffer[i] = (byte) ((value >> offset) & 0xff);
        }
        return buffer;
    }

    public static void main(String[] args) {
//        byte[] bytes = {3, 4, 0x2, 0x44, 0x44, 0x66};
//        System.out.println("macBytes2Long(bytes) = " + macBytes2Long(bytes));
//        long mac = 38028390;
//        System.out.println("long2macBytes(mac) = " + Arrays.toString(long2macBytes(mac)));


    }
}
