package db.util;

import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class FileUtils {
    private static FileUtils instance = new FileUtils();

    public static FileUtils getInstance() {
        return instance;
    }

    public static FileOutputStream openFile(String fileName) {
        File file = new File(fileName);
        // 创建文件夹
        File directory = file.getParentFile();
        if (directory != null) {
            directory.mkdirs();
        }
        FileOutputStream writer = null;
        try {
            writer = new FileOutputStream(file, true);
        } catch (FileNotFoundException e) {
            log.error("无法打开文件{}", fileName, e);
        }
        return writer;

    }

    public static boolean deleteFile(String filename) {
        File file = new File(filename);
        boolean res = true;
        if (file.exists()) {
            res = file.delete();
            log.debug("成功删除文件：{}", filename);
        }
        return res;
    }

    public static void renameFile(String oldFilename, String newFilename) {
        File oldFile = new File(oldFilename);
        File newFile = new File(newFilename);
        if (oldFile.exists() && !newFile.exists()) {  //  确保新的文件名不存在
            oldFile.renameTo(newFile);
        }
    }

    public static void combineFiles(List<String> oldFilenames, String newFilename) throws IOException {
        if (oldFilenames.size() == 1 && !oldFilenames.get(0).equals(newFilename)) {
            log.debug("将{}重命名为{}", oldFilenames.get(0), newFilename);
            FileUtils.renameFile(oldFilenames.get(0), newFilename);
        } else if (oldFilenames.size() > 1) {
            File newFile = new File(newFilename);
            FileChannel resultFileChannel = new FileOutputStream(newFile, true).getChannel();
            for (String oldFilename : oldFilenames) {
                if (oldFilename.equals(newFilename)) {
                    continue;
                }
                FileChannel blk = new FileInputStream(oldFilename).getChannel();
                resultFileChannel.transferFrom(blk, resultFileChannel.size(), blk.size());
                blk.close();
                FileUtils.deleteFile(oldFilename);
            }
            log.debug("合并文件，将{}合并为{}", oldFilenames, newFilename);
        } else {
            log.debug("待合并文件为空，无需合并");
        }
    }

    public static void appendFile(String filename, byte[] content) throws IOException {
        File file = new File(filename);
        // 创建文件夹
        File directory = file.getParentFile();
        if (directory != null) {
            directory.mkdirs();
        }

        // 追加写入
        FileOutputStream writer = new FileOutputStream(file, true);
        writer.write(content);
        writer.close();
    }

    public static void overwriteFile(String filename, byte[] content, int pos) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(filename, "rw");
        randomAccessFile.seek(pos);
        randomAccessFile.write(content);
        randomAccessFile.close();
    }

    public static void readFile(String filename, byte[] content, int pos) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(filename, "r");
        randomAccessFile.seek(pos);
        randomAccessFile.readFully(content);
        randomAccessFile.close();
    }

    public static int fileLength(String filename) {
        File file = new File(filename);
        if (file.exists() && file.isFile()) {
            return (int) file.length();
        }
        return 0;
    }

    /**
     * 获取文件夹下文件，不寻找子文件夹下的文件
     *
     * @param fileDir
     * @return
     */
    public static List<String> listFile(String fileDir) {
        List<String> list = new ArrayList<>();
        File baseFile = new File(fileDir);
        if (baseFile.isFile() || !baseFile.exists()) {
            return list;
        }
        File[] files = baseFile.listFiles();
        for (File file : files) {
            if (!file.isDirectory()) {
                list.add(fileDir + file.getName());
            }
        }
        return list;
    }


    public static void writeFile(String filename, byte[] content) throws IOException {
        File file = new File(filename);
        // 创建文件夹
        File directory = file.getParentFile();
        if (directory != null) {
            directory.mkdirs();
        }

        // 追加写入
        FileOutputStream writer = new FileOutputStream(file, false);
        writer.write(content);
        writer.close();
    }

}
