package db.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**

 * IPv4地址和int数字的互换

 * @author sun

 */

public class IpKit {

    /**

     * IPv4地址转换为int类型数字

     * @param ipv4Addr

     * @return

     */

    public static int ipToInt(String ipv4Addr) throws Exception {

// 判断是否是ip格式的

        if (!isIPv4Address(ipv4Addr))

            throw new Exception("Invalid ip address");

// 匹配数字

        Pattern pattern = Pattern.compile("\\d+");

        Matcher matcher = pattern.matcher(ipv4Addr);

        int result = 0;

        int counter = 0;

        while (matcher.find()) {

            int value = Integer.parseInt(matcher.group());

            result = (value << 8 * (3 - counter++)) | result;

        }

        return result;

    }

    /**

     * 判断是否为ipv4地址

     * @param ipv4Addr

     * @return

     */

    private static boolean isIPv4Address(String ipv4Addr) {

        String lower = "(\\d|[1-9]\\d|1\\d{2}|2[0-4]\\d|25[0-5])"; // 0-255的数字

        String regex = lower + "(\\." + lower + "){3}";

        Pattern pattern = Pattern.compile(regex);

        Matcher matcher = pattern.matcher(ipv4Addr);

        return matcher.matches();

    }

    /**

     * 将int数字转换成ipv4地址

     * @param ip

     * @return

     */

    public static String intToIp(int ip) {

        StringBuilder sb = new StringBuilder();

        int num = 0;

        boolean needPoint = false; // 是否需要加入'.'

        for (int i = 0; i < 4; i++) {

            if (needPoint) {

                sb.append('.');

            }

            needPoint = true;

            int offset = 8 * (3 - i);

            num = (ip >> offset) & 0xff;

            sb.append(num);

        }

        return sb.toString();

    }

    public static void main(String[] args) throws Exception {

        String ip = "124.202.200.166";

        System.out.println(ipToInt(ip));

        System.out.println(intToIp(2093664422));

    }

}