package db;

import db.util.Util;

import java.util.List;

/**
 * 功能描述
 *
 * @author: scott
 * @date: 2022年09月29日 12:09 PM
 */
public class PagedRangeQueryResult extends RangeQueryResult {
    int pageIndex;
    int pageSize;
    int pageCount;

    public PagedRangeQueryResult(Point point, long start, long end,
                                 long step, long lookBack,
                                 List<DBVal> dbValList,
                                 int pageIndex, int pageSize, int pageCount) {
        super(point, start, end, step, lookBack, dbValList);
        this.pageIndex = pageIndex;
        this.pageSize = pageSize;
        this.pageCount = pageCount;
    }

    public int getPageIndex() {
        return pageIndex;
    }

    public int getPageSize() {
        return pageSize;
    }

    public int getPageCount() {
        return pageCount;
    }

    public List<DBVal> getDbValList() {
        if (dbValList == null || dbValList.isEmpty()) {
            long pageStartTime = start+(pageIndex*pageSize)*step;
            return DBUtil.invalidDBValList(point, pageStartTime, step, pageSize);
        }
        return dbValList;
    }

    @Override
    public String toString() {
        try {
            return "db.PagedRangeQueryResult{" +
                    "pageIndex=" + pageIndex +
                    ", pageSize=" + pageSize +
                    ", pageCount=" + pageCount +
                    ", point=" + point +
                    ", start=" + start + String.format("(%s)", Util.uTCMilliSecondsToDateStringWithMs(start)) +
                    ", end=" + end + String.format("(%s)",Util.uTCMilliSecondsToDateStringWithMs(end)) +
                    ", step=" + step +
                    ", lookBack=" + lookBack +
                    ", count=" + count +
                    ", dbValList=" + dbValList +
                    ", success=" + success +
                    '}';
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }
}
