package db;

import com.alibaba.fastjson.JSONObject;

import java.util.*;

public class Point {
    private String metricName; // 数值型度量名
    private Map<String, String> tags;

    public Point(String metricName, Map<String, String> tags) {
        this.metricName = metricName;
        this.tags = tags;
    }

    public Point() {
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Point point = (Point) o;
        return metricName.equals(point.metricName) && tags.equals(point.tags);
    }

    @Override
    public int hashCode() {
        return Objects.hash(metricName, tags);
    }

    public static Point fromJSONObject(JSONObject jsonObject) {
        Point point = new Point();
        HashMap<String, String> tags = new HashMap<>();
        point.setMetricName(jsonObject.getString("__name__"));

        for (String tagName : jsonObject.keySet()) {
            if(tagName.equals("__name__")) continue;
            tags.put(tagName, jsonObject.get(tagName).toString());
        }
        point.setTags(tags);
        return point;
    }

    public String getMetricName() {
        return metricName;
    }

    public void setMetricName(String metricName) {
        this.metricName = metricName;
    }

    public Map<String, String> getTags() {
        return tags;
    }

    public void setTags(HashMap<String, String> tags) {
        this.tags = tags;
    }

    @Override
    public String toString() {
        return "db.Point{" +
                "metricName='" + metricName + '\'' +
                ", tags=" + tags +
                '}';
    }

    public String calPointName(String metricNamePrefix) {
        StringBuffer sb = new StringBuffer();
        sb.append(metricNamePrefix);
        sb.append(this.getMetricName());
        if(this.getTags()==null || this.getTags().isEmpty()) {
            return sb.toString();
        }
        sb.append("{");

        String tagName, tagValue;
        Vector<String> tagNames = new Vector<>();
        tagNames.addAll(this.getTags().keySet());
        for(int i=0;i<tagNames.size();i++) {
            if(i!=0) {
                sb.append(",");
            }

            tagName = tagNames.elementAt(i);
            tagValue = this.getTags().get(tagName);
            sb.append(String.format("%s=\"%s\"", tagName, tagValue));
        }
        sb.append("}");
        return sb.toString();
    }

    public static String calQueryStringForMultiPoint(String metricNamePrefix, String metricName,
                                      HashMap<String, String> commonTagValuePairs,
                                      String tagNameWithDifferentValue,
                                      List<String> tagValues) {
        StringBuffer sb = new StringBuffer();
        sb.append(metricNamePrefix);
        sb.append(metricName);
        sb.append("{");

        String tagName, tagValue;
        Vector<String> tagNames = new Vector<>();
        tagNames.addAll(commonTagValuePairs.keySet());
        for(int i=0;i<tagNames.size();i++) {
            if(i!=0) {
                sb.append(",");
            }

            tagName = tagNames.elementAt(i);
            tagValue = commonTagValuePairs.get(tagName);
            sb.append(String.format("%s=\"%s\"", tagName, tagValue));
        }

        if(!tagNames.isEmpty()) {
            sb.append(",");
        }

        StringBuffer tagValuesBuffer = new StringBuffer();
        for(int j=0;j<tagValues.size();j++) {
            if(j!=0)
                tagValuesBuffer.append("|");
            tagValuesBuffer.append(tagValues.get(j));
        }

        if(!tagValues.isEmpty()) {
            sb.append(String.format("%s=~\"%s\"", tagNameWithDifferentValue, tagValuesBuffer.toString()));
        }
        sb.append("}");
        return sb.toString();
    }

    public static String calQueryStringForMultiPoint(String metricNamePrefix, String metricName,
                                      HashMap<String, String> commonTagValuePairs) {
        StringBuffer sb = new StringBuffer();
        sb.append(metricNamePrefix);
        sb.append(metricName);
        sb.append("{");

        String tagName, tagValue;
        Vector<String> tagNames = new Vector<>();
        tagNames.addAll(commonTagValuePairs.keySet());
        for(int i=0;i<tagNames.size();i++) {
            if(i!=0) {
                sb.append(",");
            }

            tagName = tagNames.elementAt(i);
            tagValue = commonTagValuePairs.get(tagName);
            sb.append(String.format("%s=\"%s\"", tagName, tagValue));
        }
        sb.append("}");
        return sb.toString();
    }

    /*
     * {__name__=\"%s\",pointName=\"%s\"," +
                        "status=\"%d\"}
     */
    public String calMatchPointName(String metricNamePrefix) {
        StringBuffer sb = new StringBuffer();
        sb.append("{");
//        sb.append(String.format("%s=\"%s\"", "__name__", metricNamePrefix+this.getMetricName()));
        sb.append(String.format("%s=~\"%s\"", "__name__", metricNamePrefix+this.getMetricName()));

        String tagName, tagValue;
        Vector<String> tagNames = new Vector<>();
        tagNames.addAll(this.getTags().keySet());
        for(int i=0;i<tagNames.size();i++) {
            tagName = tagNames.elementAt(i);
            tagValue = this.getTags().get(tagName);
            sb.append(",");
            sb.append(String.format("%s=\"%s\"", tagName, tagValue));
        }
        sb.append("}");
        return sb.toString();
    }


}
