package db.entity;

import db.Globals;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.NoArgsConstructor;

@Data
@NoArgsConstructor
public class LogItem {
    public static final short DEFAULT_LOG_ITEM_TYPE = 0x55;
    int magic = Globals.LOG_ITEM_MAGIC;
    short type;
    short length;
    long stamp;
    byte[] rawPacket;

    public LogItem(short type, long stamp, byte[] rawPacket) {
        this.type = type;
        this.length = (short) (16 + rawPacket.length);
        this.stamp = stamp;
        this.rawPacket = rawPacket;
    }

    public byte[] getPayload() {
        ByteBuf buf = Unpooled.buffer(length);
        buf.writeInt(magic);
        buf.writeShort(type);
        buf.writeShort(length);
        buf.writeLong(stamp);
        buf.writeBytes(rawPacket);
        return buf.array();
    }

    public void setPayload(byte[] buffer) {
        ByteBuf buf = Unpooled.wrappedBuffer(buffer);

        magic = buf.readInt();
        type = buf.readShort();
        length = buf.readShort();
        stamp = buf.readLong();
        rawPacket = new byte[length];
        buf.readBytes(rawPacket);
    }

    public boolean setHeader(byte[] buffer) {
        ByteBuf buf = Unpooled.wrappedBuffer(buffer);

        magic = buf.readInt();
        if(magic != Globals.LOG_ITEM_MAGIC) {
            return false;
        }
        type = buf.readShort();
        length = buf.readShort();
        stamp = buf.readLong();
        return true;
    }

    public void setBody(byte[] buffer) {
        rawPacket = buffer;
    }
}
