package db.mapper;

import db.entity.LogFileMeta;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;

@Mapper
public interface LogFileMetaMapper {

    @Insert("insert into log_file_meta(id,file_name,min_time,max_time,count,valid,save_disk,save_tsdb," +
            "save_disk_stamp,save_tsdb_stamp,wal_stamp) values (#{id},#{fileName},#{minTime},#{maxTime},#{count}," +
            "#{valid},#{saveDisk},#{saveTsdb},#{saveDiskStamp},#{saveTsdbStamp},#{walStamp})")
    public int insert(LogFileMeta logFileMeta);

    @Update("update log_file_meta set file_name = #{fileName}, min_time = #{minTime}, max_time = #{maxTime}," +
            "count = #{count}, valid = #{valid}, save_disk = #{saveDisk}, save_tsdb = #{saveTsdb}, save_disk_stamp = " +
            "#{saveDiskStamp}, save_tsdb_stamp = #{saveTsdbStamp}, wal_stamp = #{walStamp} where id = #{id}")
    public int update(LogFileMeta logFileMeta);

    @Select("select * from log_file_meta")
    public List<LogFileMeta> getLogFileMeta();

    /**
     * 获取时间范围内所有（包括无效）的log文件
     * @param start
     * @param end
     * @return
     */
    @Select("select * from log_file_meta where not (min_time > #{arg1} or max_time < #{arg0})")
    public List<LogFileMeta> getLogFileMetaBetweenTime(long start, long end);

    /**
     * 获取时间范围内有效的log文件
     * @param start
     * @param end
     * @return
     */
    @Select("select * from log_file_meta where not (min_time > #{arg1} or max_time < #{arg0}) and valid=1")
    public List<LogFileMeta> getValidLogFileMetaBetweenTime(long start, long end);

    /**
     * 获取最新的一条log文件
     * @return
     */
    @Select("select * from log_file_meta order by id desc limit 1")
    public LogFileMeta getLastLogFileMeta();

    /**
     * 获取一条有效的未填充满的log文件
     * @param countLimit 限制已填充大小（小于等于该大小）
     * @return
     */
    @Select("select * from log_file_meta where valid=1 and save_disk=1 and save_tsdb=1 and count<=#{countLimit} limit" +
            " 1")
    public LogFileMeta getValidUnfilledLogFileMeta(int countLimit);

    @Select("select * from log_file_meta where id = #{id}")
    public LogFileMeta getLogFileMetaById(long id);

    /**
     * 检查过期文件，仅查找有效文件
     * @param expireTime
     * @return
     */
    @Select("select * from log_file_meta where max_time < #{expireTime} and valid = 1")
    public List<LogFileMeta> getValidExpiredLogFileMeta(long expireTime);

    /**
     * 标记指定文件无效
     * @param id
     * @return
     */
    @Select("update log_file_meta set valid = 0 where id = #{id}")
    public List<LogFileMeta> markInvalidById(long id);

    /**
     * 标记过期文件无效
     * @param expireTime
     * @return
     */
    @Select("update log_file_meta set valid = 0 where max_time < #{expireTime} and valid = 1")
    public List<LogFileMeta> markValidExpiredLogFileMetaAsInvalid(long expireTime);

    /**
     * 查找保存超时的文件
     * @param expireTime
     * @return
     */
    @Select("select * from log_file_meta where (save_disk=0 and save_disk_stamp < #{expireTime}) or (save_tsdb=0 and " +
            "save_tsdb_stamp < #{expireTime}) and valid = 1")
    public List<LogFileMeta> getValidTimeOutLogFileMeta(long expireTime);

    /**
     * 查找保存disk超时的文件
     * @param expireTime
     * @return
     */
    @Select("select * from log_file_meta where save_disk=0 and save_disk_stamp < #{expireTime} and valid = 1")
    public List<LogFileMeta> getValidDiskTimeOutLogFileMeta(long expireTime);

     /**
     * 查找保存tsdb超时的文件
     * @param expireTime
     * @return
     */
    @Select("select * from log_file_meta where save_tsdb=0 and save_tsdb_stamp < #{expireTime} and valid = 1")
    public List<LogFileMeta> getValidTsdbTimeOutLogFileMeta(long expireTime);

}
