package db.controller;

import db.dto.ProbeDto;
import db.entity.ResponseDto;
import db.entity.Probe;
import db.service.ProbeService;
import db.util.ResponseUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@Slf4j
@RequestMapping("/manage")
public class ProbeController {

    @Autowired
    private ProbeService probeService;

    @RequestMapping("/listen")
    public String listenConfig(@RequestParam String type, @RequestParam String mode) {

        log.info("收到监听请求，type={}，mode={}", type, mode);

        probeService.listenConfig(type, mode);

        return "成功";
    }

    @RequestMapping("/scan")
    public String scanProbes() {
        log.info("==> [api扫描探针] 收到扫描请求");

        probeService.scanProbes();

        String res = "已发送扫描请求";
        log.info("<== [api扫描探针] {}", res);
        return res;
    }

    @RequestMapping("/unlisten")
    public String unlisten(@RequestParam String probeIP) {
        log.info("==> [api取消数据监听] 收到请求");

        probeService.sendUnListenPackage(probeIP);

        String res = "已发送取消数据监听";
        log.info("<== [api取消数据监听] {}", res);
        return res;
    }

    @RequestMapping("/genData")
    public String genData(@RequestParam Integer dataCount) {
        log.info("==> [api模拟插入随机数据] 收到请求");

        long startTime = System.currentTimeMillis();
        probeService.genData(dataCount);
        long endTime = System.currentTimeMillis();

        String res = String.format("已模拟插入随机数据%d条", dataCount);
        log.info("<== [api模拟插入随机数据] 用时：{}ms，内容：{}", endTime-startTime, res);
        return res;
    }


    @RequestMapping("/probe")
    public ResponseDto<List<ProbeDto>> queryProbes() {
        log.info("==> [api查看所有探针] 收到查看探针请求");

        List<ProbeDto> probes = probeService.queryProbes();

        ResponseDto<List<ProbeDto>> ret;

        ret = ResponseUtils.success(probes);

        log.info("<== [api查看所有探针] {}", ret);
        return ret;
    }

    @RequestMapping("/probe/{probeId}")
    public ResponseDto<ProbeDto> queryProbes(@PathVariable short probeId) {
        log.info("==> [api查看探针详情] 收到查看探针请求");

        ProbeDto probe = probeService.queryProbes(probeId);

        ResponseDto<ProbeDto> ret;

        ret = ResponseUtils.success(probe);

        log.info("<== [api查看探针详情] {}", ret);
        return ret;
    }
}
