package db.dto;

import com.fasterxml.jackson.annotation.JsonProperty;
import db.entity.Probe;
import db.util.ConvertUtils;
import lombok.Data;
import lombok.NoArgsConstructor;

@Data
@NoArgsConstructor
public class ProbeDto {
    private short id;
    private String mac;
    private String ip;
    private String mask;
    private String clockMode;
    @JsonProperty("eE1LinkState")
    private String eE1LinkState;
    @JsonProperty("eE1LinkDeviceType")
    private String eE1LinkDeviceType;
    @JsonProperty("eE1LinkDevicePort")
    private byte eE1LinkDevicePort;
    @JsonProperty("eE1LinkDeviceId")
    private short eE1LinkDeviceId;
    @JsonProperty("eE2LinkState")
    private String eE2LinkState;
    @JsonProperty("eE2LinkDeviceType")
    private String eE2LinkDeviceType;
    @JsonProperty("eE2LinkDevicePort")
    private byte eE2LinkDevicePort;
    @JsonProperty("eE2LinkDeviceId")
    private short eE2LinkDeviceId;
    private String e0linkState;
    private String e1LinkState;
    private String e2LinkState;
    private String e3LinkState;
    private String e4LinkState;
    private String e5LinkState;
    private String e6LinkState;
    private String e7LinkState;
    private String clockSource;

    public ProbeDto(Probe probe) {
        this.id = probe.getId();
        this.mac = ConvertUtils.longMac2String(probe.getMac());
        this.ip = ConvertUtils.ip2String(probe.getIp());
        this.mask = ConvertUtils.ip2String(probe.getMask());
        this.clockMode = convertClockMode(probe.getClockMode());
        this.eE1LinkState = convertLinkState(probe.getEE1LinkState());
        this.eE1LinkDeviceType = convertLinkDeviceType(probe.getEE1LinkDeviceType());
        this.eE1LinkDevicePort = probe.getEE1LinkDevicePort();
        this.eE1LinkDeviceId = probe.getEE1LinkDeviceId();
        this.eE2LinkState = convertLinkState(probe.getEE2LinkState());
        this.eE2LinkDeviceType = convertLinkDeviceType(probe.getEE2LinkDeviceType());
        this.eE2LinkDevicePort = probe.getEE2LinkDevicePort();
        this.eE2LinkDeviceId = probe.getEE2LinkDeviceId();
        this.e0linkState = convertLinkState(probe.getE0linkState());
        this.e1LinkState = convertLinkState(probe.getE1LinkState());
        this.e2LinkState = convertLinkState(probe.getE2LinkState());
        this.e3LinkState = convertLinkState(probe.getE3LinkState());
        this.e4LinkState = convertLinkState(probe.getE4LinkState());
        this.e5LinkState = convertLinkState(probe.getE5LinkState());
        this.e6LinkState = convertLinkState(probe.getE6LinkState());
        this.e7LinkState = convertLinkState(probe.getE7LinkState());
        this.clockSource = convertClockSource(probe.getClockSource());
    }

    private String convertClockMode(byte clockMode) {
        switch (clockMode) {
            case 0x3A:
                return "IRIG_BIRIG_B";
            case 0x35:
                return "PPS";
            default:
                return "";
        }
    }

    private String convertLinkState(byte linkState) {
        switch (linkState) {
            case (byte) 0xAA:
                return "Link up";
            case 0x55:
                return "Link down";
            default:
                return "";
        }
    }

    private String convertLinkDeviceType(byte linkDeviceType) {
        switch (linkDeviceType) {
            case 0x0:
                return "未连接或未识别交换机";
            case 0x1:
                return "TAN核心交换机";
            case 0x2:
                return "TAN接入交换机";
            case 0x3:
                return "探针交换机";
            default:
                return "";
        }
    }

    private String convertClockSource(byte clockSource) {
        switch (clockSource) {
            case (byte) 0xAA:
                return "有";
            default:
                return "无";
        }
    }

}
