package db.netty;

import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioDatagramChannel;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.PostConstruct;

@Slf4j
public class NettyClient {

    private EventLoopGroup group = new NioEventLoopGroup();

    private Channel channel;

    @PostConstruct
    public void start() throws InterruptedException {
        log.info("开始创建client");

        Bootstrap bootstrap = new Bootstrap();
        bootstrap.group(group)
                .channel(NioDatagramChannel.class)
                .option(ChannelOption.SO_BROADCAST, true)
                .handler(new ChannelInitializer<NioDatagramChannel>() {
                    @Override
                    public void initChannel(final NioDatagramChannel ch) throws Exception {

                        ChannelPipeline p = ch.pipeline();
                        p.addLast(new Encoder(), new Decoder());
                        p.addLast(new NettyClientHandler());
                    }
                });

        channel = bootstrap.bind(1200).sync().channel();
        log.info("创建client成功");
    }

    public static void main(String[] args) throws InterruptedException {
        NettyClient client = new NettyClient();
        client.start();
        log.info("启动 Client");
    }


}
