package db.netty;

import db.entity.packages.BasePackage;
import db.entity.packages.DataResponsePackage;
import db.entity.packages.ListenResponsePackage;
import db.entity.packages.ScanResponsePackage;
import db.handler.PackageHandler;
import db.util.SpringUtils;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;


@Slf4j
@Data
public class NettyServerHandler extends SimpleChannelInboundHandler<BasePackage> {
    private boolean useSeperateThread = false;

    private PackageHandler packageHandler;

    protected void processPackage(BasePackage basePackage) {
        if (basePackage instanceof DataResponsePackage) {
            packageHandler.handleDataResponsePackage((DataResponsePackage) basePackage);
        } else if (basePackage instanceof ScanResponsePackage) {
            packageHandler.handleScanResponsePackage((ScanResponsePackage) basePackage);
        } else if (basePackage instanceof ListenResponsePackage) {
            packageHandler.handleListenResponsePackage((ListenResponsePackage) basePackage);
        }
        // TODO 在这里扩展数据报文的处理
    }

    @Override
    protected void channelRead0(ChannelHandlerContext arg0, BasePackage basePackage) throws Exception {
        log.debug("channelRead0 收到数据:{}", basePackage);

        if (packageHandler == null) {
            packageHandler = SpringUtils.getBean(PackageHandler.class);
        }

        if (useSeperateThread) {
            new Thread(() -> {
                processPackage(basePackage);
            }).start();
        } else {
            processPackage(basePackage);
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        log.info("NettyServerHandler exceptionCaught:{}", cause);
        cause.printStackTrace();
        ctx.fireExceptionCaught(cause);
    }
}
