package db.service;

import db.Globals;
import db.data.DataGenerator;
import db.dto.ProbeDto;
import db.entity.Probe;
import db.entity.packages.DataResponsePackage;
import db.entity.packages.ListenRequestPackage;
import db.entity.packages.ScanRequestPackage;
import db.handler.PackageHandler;
import db.netty.NettyServer;
import db.util.IpKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class ProbeService {

    private Map<Short, Probe> probes = new Hashtable<>();

    @Autowired
    private NettyServer server;

    @Autowired
    private PackageHandler packageHandler;

    public Map<Short, Probe> getProbes() {
        return probes;
    }

    /**
     * 创建mock探针
     */
//    @PostConstruct
    private void initMockProbe() {
        Probe a = new Probe((short) 0x0A,
                0x5e43bbbb000aL, 0x7F000001, 0xFFFF0000, (byte) 0x35, (byte) 0xAA, (byte) 0x1, (byte) 31,
                (short) 32, (byte) 0x55, (byte) 0x2, (byte) 35, (short) 36, (byte) 0xAA, (byte) 0xAA,
                (byte) 0xAA, (byte) 0xAA, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0xAA);
        Probe b = new Probe((short) 0x0B,
                0x6c5a33d92137L, 0x7F000002, 0xFFFF0000, (byte) 0x3A, (byte) 0xAA, (byte) 0x0, (byte) 31,
                (short) 32, (byte) 0x55, (byte) 0x3, (byte) 35, (short) 36, (byte) 0xAA, (byte) 0xAA,
                (byte) 0xAA, (byte) 0xAA, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0xAA);
        probes.put((short) 0x0A, a);
        probes.put((short) 0x0B, b);
    }

    public void listenConfig(String typeStr, String modeStr) {

//        byte mode;
//        if (modeStr.equalsIgnoreCase("all")) {
//            mode = (byte) 0xFF;
//        } else if (modeStr.equalsIgnoreCase("black")) {
//            mode = 0x55;
//        } else if (modeStr.equalsIgnoreCase("white")) {
//            mode = (byte) 0xAA;
//        } else {
//            log.error("未识别的监听请求模式：{}", modeStr);
//            return;
//        }
//
//
//        if (typeStr.equalsIgnoreCase("Start")) {
//            listenPackageSender.startListen(mode);
//        } else {
//            listenPackageSender.cancelListen(mode);
//        }
    }

    public void scanProbes() {
        // 广播发送报文

        ScanRequestPackage scanRequestPackage = new ScanRequestPackage();

        scanRequestPackage.setToHostPort(Globals.BROADCAST_HOST, Globals.BROADCAST_PORT);

        // 仅发送一次
        scanRequestPackage.setSendTimes(1);

        log.info("<== [设备扫描] 发送{} BROADCAST_HOST:{}  BROADCAST_PORT:{}", scanRequestPackage,
                Globals.BROADCAST_HOST, Globals.BROADCAST_PORT);
        server.send(scanRequestPackage);
    }

    public List<ProbeDto> queryProbes() {

        //------------------
        // 创建mock数据
        Probe a = new Probe((short) 0x0A,
                0x5e43bbbb000aL, 0x7F000001, 0xFFFF0000, (byte) 0x35, (byte) 0xAA, (byte) 0x1, (byte) 31,
                (short) 32, (byte) 0x55, (byte) 0x2, (byte) 35, (short) 36, (byte) 0xAA, (byte) 0xAA,
                (byte) 0xAA, (byte) 0xAA, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0xAA);
        Probe b = new Probe((short) 0x0B,
                0x6c5a33d92137L, 0x7F000002, 0xFFFF0000, (byte) 0x3A, (byte) 0xAA, (byte) 0x0, (byte) 31,
                (short) 32, (byte) 0x55, (byte) 0x3, (byte) 35, (short) 36, (byte) 0xAA, (byte) 0xAA,
                (byte) 0xAA, (byte) 0xAA, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0xAA);
        List<ProbeDto> res = new ArrayList<>();
        res.add(a.toDto());
        res.add(b.toDto());

        // 以上为mock代码
        //------------------
        // 以下为原始正常代码
//        Collection<Probe> valueCollection = probes.values();
//        List<ProbeDto> res = new ArrayList<>();
//        for (Probe probe : valueCollection) {
//            res.add(probe.toDto());
//        }
        return res;
    }

    public ProbeDto queryProbes(short probeId) {
        return probes.get(probeId).toDto();
    }

    @Scheduled(fixedDelay = 4000) // 毫秒
    private void sendListenPackage() {
        for (Probe probe : probes.values()) {
            // 发送监听报文
            ListenRequestPackage listenRequestPackage = new ListenRequestPackage(Globals.OPERATION_LISTEN_LOGIN, (byte) 0xFF);

            listenRequestPackage.setToHostPort(probe.getIp(), Globals.BROADCAST_PORT);

            log.info("向 probe:{} 发送 listenRequestPackage:{}", probe.getIp(), listenRequestPackage);
            server.send(listenRequestPackage);
        }
    }

    @Scheduled(fixedDelay = 60000) // 毫秒
    private void sendScanPackage() {
        // 发送监听报文
        ScanRequestPackage scanRequestPackage = new ScanRequestPackage();

        scanRequestPackage.setToHostPort(Globals.BROADCAST_HOST, Globals.BROADCAST_PORT);

        log.info("广播发送 scanRequestPackage:{}", scanRequestPackage);
        server.send(scanRequestPackage);
    }

    public void sendUnListenPackage(String probeIP) {
        int ipInteger;

        try {
            ipInteger = IpKit.ipToInt(probeIP);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        short probeID = -1;
        for (Probe probe : probes.values()) {
            if (probe.getIp() == ipInteger) {
                probeID = probe.getId();
                log.info("在probe列表中查找到probe:{} probeIP:{}", probe, probeIP);
            }
        }

        if (probeID == -1) {
            log.info("在probe列表中未查找到probeIP:{}", probeIP);
        } else {
            log.info("在probe列表中查找到probeIP:{} ", probeIP);
            probes.remove(probeID);
            log.info("在probe列表中删除probeID:{} ", probeID);
        }

        // 发送取消监听报文
        ListenRequestPackage unlistenRequestPackage = new ListenRequestPackage(Globals.OPERATION_LISTEN_CANCEL, (byte) 0xFF);

        unlistenRequestPackage.setToHostPort(probeIP, Globals.BROADCAST_PORT);

        log.info("向probe:{} 发送取消listenRequestPackage:{}", probeIP, unlistenRequestPackage);
        server.send(unlistenRequestPackage);
    }


    public void genData(Integer dataCount) {
        DataGenerator dataGenerator = new DataGenerator();
        List<DataResponsePackage> packages = dataGenerator.generate(dataCount);
        for (DataResponsePackage dataPackage: packages) {
            packageHandler.handleDataResponsePackage(dataPackage);
        }
    }
}
