package db;

import db.data.DataGenerator;
import db.entity.packages.DataResponsePackage;
import db.netty.Decoder;
import db.netty.Encoder;
import db.simulator.NettyClientHandler;
import db.util.Config;
import db.util.SpentTimeCalculator;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioDatagramChannel;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.PostConstruct;
import java.util.List;

@Slf4j
public class Probe {

    private EventLoopGroup group = new NioEventLoopGroup();

    private Channel channel;

    @PostConstruct
    public void start() throws InterruptedException {
        log.info("开始创建netty client");

        Bootstrap bootstrap = new Bootstrap();
        NettyClientHandler nettyClientHandler = new NettyClientHandler();
        nettyClientHandler.setProbeID(Config.getInstance().probeID());

        bootstrap.group(group)
                .channel(NioDatagramChannel.class)
                .option(ChannelOption.SO_BROADCAST, true)
                .handler(new ChannelInitializer<NioDatagramChannel>() {
                    @Override
                    public void initChannel(final NioDatagramChannel ch) throws Exception {

                        ChannelPipeline p = ch.pipeline();
                        p.addLast(new Encoder(), new Decoder());
                        p.addLast(nettyClientHandler);
                    }
                });

//        channel = bootstrap.bind(1200).sync().channel();
        log.info("创建模拟探针成功");

        // 客户端开启
        String host = Config.getInstance().blackboxServer();
        int port = Config.getInstance().blackboxServerPort();

        log.info("连接blackboxserver host:{} port:{}", host, port);
        ChannelFuture cf = bootstrap.connect(host, port).sync();
        Channel clientChannel = cf.channel();
        int packetNumPerSecond = Config.getInstance().packetNumPerSecond();

        log.info("开始向blackboxserver发送数据 host:{} port:{}", host, port);
        DataGenerator dataGenerator = new DataGenerator();
        List<DataResponsePackage> packages;
        SpentTimeCalculator spentTimeCalculator = SpentTimeCalculator.create();
        SpentTimeCalculator genDataAndSendTimer = SpentTimeCalculator.create();
        int dataIndex = 0;
        long totalCount = 0;
        while (true) {
            genDataAndSendTimer.begin();
            dataIndex++;
            log.info("[{}] 生成数据 count:{}", dataIndex, packetNumPerSecond);
            spentTimeCalculator.begin();
            packages = dataGenerator.generate(packetNumPerSecond);
            spentTimeCalculator.end();
            log.info("[{}] 生成数据 count:{}，花费时间:{}ms", dataIndex, packetNumPerSecond,
                    spentTimeCalculator.getSpendTime());
            log.info("[{}] 发送数据 count:{}", dataIndex, packetNumPerSecond);
            for(DataResponsePackage p :packages) {
                clientChannel.writeAndFlush(p);
            }
            genDataAndSendTimer.end();
            log.info("[{}] 生成和发送数据 count:{}，花费时间:{}ms", dataIndex, packetNumPerSecond,
                    genDataAndSendTimer.getSpendTime());
            totalCount += packetNumPerSecond;
            log.info("[{}] 目前生成和发送数据总数 totalCount:{}", dataIndex, totalCount);

            if(genDataAndSendTimer.getSpendTime()<1000) {
                Thread.sleep(1000-genDataAndSendTimer.getSpendTime());
            }
        }
    }

    public static void main(String[] args) throws InterruptedException {
        if(args.length>=1) {
            String configFile = args[0];
            Config.setConfigFile(configFile);
        }
        log.info("configFile:{}", Config.getConfigFile());

        Probe probe = new Probe();
        probe.start();
        log.info("启动模拟探针");
    }
}
