package db.simulator;

import db.Globals;
import db.entity.packages.*;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Data
public class NettyClientHandler extends SimpleChannelInboundHandler<BasePackage> {
    boolean scanRequestReceived = false;
    boolean listenRequestReceived = false;
    int probeID;

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, BasePackage basePackage) throws Exception {
        log.info("Probe收到数据:{}", basePackage);
        if (basePackage instanceof ScanRequestPackage) {
            scanRequestReceived = true;
            sendScanResponse(ctx, basePackage.getFromHost(), basePackage.getFromPort());
        } else if (basePackage instanceof ListenRequestPackage) {
            listenRequestReceived = true;
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        cause.printStackTrace();
        ctx.close();
    }

    public void sendScanResponse(ChannelHandlerContext ctx, String toHost, int toPort) {
        log.info("Probe发送sendScanResponse");

        ScanResponsePackage scanResponsePackage = new ScanResponsePackage(Globals.RESPONSE_SUCCESS, (short) probeID,
                0x5e43bbbb000aL, 0x7F000001, 0xFFFF0000, (byte) 0x3A, (byte) 0xAA, (byte) 0x2, (byte) 29,
                (short) 30, (byte) 0x55, (byte) 0x1, (byte) 31, (short) 32, (byte) 0xAA, (byte) 0xAA,
                (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0x55, (byte) 0xAA);
        scanResponsePackage.setToHostPort(toHost, toPort);
        send(ctx, scanResponsePackage);
    }

    public void send(ChannelHandlerContext ctx, BasePackage basePackage) {
        ctx.writeAndFlush(basePackage);
    }
}
