package db.entity.packages;

import db.Globals;
import db.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

@Data
@Slf4j
@ToString(callSuper = true)
public class BaseInfoWriteRequestPackage extends BasePackage {

    private long mac;
    private int ip;
    private int mask;
    private byte clockMode;

    public BaseInfoWriteRequestPackage() {
        super(Globals.TYPE_REQUEST, Globals.NONE, Globals.OPERATION_WRITE, Globals.PACKAGE_BASE_INFO, (short) 18);
    }

    @Override
    public void setPayload(ByteBuf buf) {
        buf.readByte();
        mac = buf.readUnsignedInt() << 16 + buf.readUnsignedShort();
        ip = buf.readInt();
        mask = buf.readInt();
        buf.readByte();
        buf.readByte();
        clockMode = buf.readByte();
    }

    @Override
    public ByteBuf getPayload() {
        ByteBuf buf = Unpooled.buffer(18);
        buf.writeByte(0);
        buf.writeInt((int) (mac >> 16));
        buf.writeShort((int) mac);
        buf.writeInt(ip);
        buf.writeInt(mask);
        buf.writeByte(0);
        buf.writeByte(0);
        buf.writeByte(clockMode);
        return buf;
    }

    @Override
    public String toString() {
        return "BaseInfoWriteRequestPackage{" +
                "mac=" + ConvertUtils.longMac2String(mac) +
                ", ip=" + ConvertUtils.ip2String(ip) +
                ", mask=" + ConvertUtils.ip2String(mask) +
                ", clockMode=" + Integer.toHexString(clockMode) +
                "} " + super.toString();
    }
}
