package db.entity.packages;

import db.Globals;
import db.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.text.ParseException;
import java.util.concurrent.Delayed;
import java.util.concurrent.TimeUnit;

@Data
@NoArgsConstructor
public abstract class BasePackage implements Delayed {

    private String fromHost;
    private Integer fromPort;
    private String toHost;
    private Integer toPort;
    private int sendTimes = 1; // 次数，为1是只发送一次
    private Long sendStamp; // 发送时刻

    private int magic = Globals.PROBE_MAGIC;
    private int version= Globals.VERSION;

    private short operationType;
    private short responseCode;
    private short operationCode;
    private short packageCode;
    private short packageLength;

    @Override
    public long getDelay(TimeUnit unit) {
        return sendStamp - System.currentTimeMillis();
    }

    @Override
    public int compareTo(Delayed o) {
        BasePackage task = (BasePackage) o;
        long diff = this.sendStamp - task.sendStamp;
        if (diff <= 0) {
            return -1;
        } else {
            return 1;
        }
    }

    public void setFromHostPort(String fromHost, Integer fromPort) {
        this.fromHost = fromHost;
        this.fromPort = fromPort;
    }

    public void setToHostPort(String toHost, Integer toPort) {
        this.toHost = toHost;
        this.toPort = toPort;
    }

    public void setToHostPort(Integer toIp, Integer toPort) {
        this.toHost = ConvertUtils.ip2String(toIp);
        this.toPort = toPort;
    }

    public BasePackage(short operationType, short responseCode, short operationCode, short packageCode, short packageLength) {
        this.operationType = operationType;
        this.responseCode = responseCode;
        this.operationCode = operationCode;
        this.packageCode = packageCode;
        this.packageLength = packageLength;
    }

    public void setHeader(ByteBuf buf) {
        magic = buf.readInt(); // skip magic
        version = buf.readInt();
        operationType = buf.readShort();
        responseCode = buf.readShort();
        operationCode = buf.readShort();
        packageCode = buf.readShort();
        packageLength = buf.readShort();
    }

    public abstract void setPayload(ByteBuf buf) throws ParseException;

    public ByteBuf getHeader() {
        ByteBuf buf = Unpooled.buffer(18);
        buf.writeInt(magic);
        buf.writeInt(version);
        buf.writeShort(operationType);
        buf.writeShort(responseCode);
        buf.writeShort(operationCode);
        buf.writeShort(packageCode);
        buf.writeShort(packageLength);

        return buf;
    }

    public abstract ByteBuf getPayload();

    @Override
    public String toString() {
        return "BasePackage{" +
                "fromHost='" + fromHost + '\'' +
                ", fromPort=" + fromPort +
                ", toHost='" + toHost + '\'' +
                ", toPort=" + toPort +
                ", sendTimes=" + sendTimes +
                ", sendStamp=" + sendStamp +
                ", magic=" + Integer.toHexString(magic) +
                ", version=" + Integer.toHexString(version) +
                ", operationType=" + Integer.toHexString(operationType) +
                ", responseCode=" + Integer.toHexString(responseCode) +
                ", operationCode=" + Integer.toHexString(operationCode) +
                ", packageCode=" + Integer.toHexString(packageCode) +
                ", packageLength=" + packageLength +
                '}';
    }
}
