package db.entity.packages;

import db.Globals;
import db.entity.ClientInfo;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Data
@Slf4j
@NoArgsConstructor
@ToString(callSuper = true)
public class ClientReadResponsePackage extends BasePackage {

    private byte port;
    private List<ClientInfo> clientInfos;

    public ClientReadResponsePackage(short responseCode) {
        super(Globals.TYPE_RESPONSE, responseCode, Globals.OPERATION_READ, Globals.PACKAGE_CLIENT_INFO, (short) 0);
    }

    @Override
    public void setPayload(ByteBuf buf) {
        port = buf.readByte();
        ClientInfo clientInfo = new ClientInfo();
        clientInfo.setConfirm(buf.readByte());
        buf.readByte(); // byte1
        clientInfo.setMac(buf.readUnsignedInt() << 16 + buf.readUnsignedShort());
        clientInfo.setIp(buf.readInt());
        buf.readByte();
        buf.readByte();
        buf.readByte();
        buf.readByte();
        clientInfos.add(clientInfo);
    }

    @Override
    public ByteBuf getPayload() {
        int len = clientInfos.size() * 16 + 1;
        super.setPackageLength((short) len);
        ByteBuf buf = Unpooled.buffer(len);
        buf.writeByte(port);
        for (int i = 0; i < clientInfos.size(); i++) {
            buf.writeByte(clientInfos.get(i).getConfirm());
            buf.writeByte(0);
            buf.writeInt((int) (clientInfos.get(i).getMac() >> 16));
            buf.writeShort((int) clientInfos.get(i).getMac());
            buf.writeInt(clientInfos.get(i).getIp());
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeByte(0);
        }
        return buf;
    }

}
