package db.entity.packages;

import db.Globals;
import db.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

@Data
@Slf4j
@NoArgsConstructor
@ToString(callSuper = true)
public class ScanResponsePackage extends BasePackage {

    private short id;
    private long mac;
    private int ip;
    private int mask;
    private byte clockMode;
    private byte eE1LinkState;
    private byte eE1LinkDeviceType;
    private byte eE1LinkDevicePort;
    private short eE1LinkDeviceId;
    private byte eE2LinkState;
    private byte eE2LinkDeviceType;
    private byte eE2LinkDevicePort;
    private short eE2LinkDeviceId;
    private byte e0linkState;
    private byte e1LinkState;
    private byte e2LinkState;
    private byte e3LinkState;
    private byte e4LinkState;
    private byte e5LinkState;
    private byte e6LinkState;
    private byte e7LinkState;
    private byte clockSource;

    public ScanResponsePackage(short responseCode, short id, long mac, int ip, int mask, byte clockMode,
                               byte eE1LinkState, byte eE1LinkDeviceType, byte eE1LinkDevicePort, short eE1LinkDeviceId,
                               byte eE2LinkState, byte eE2LinkDeviceType, byte eE2LinkDevicePort, short eE2LinkDeviceId,
                               byte e0linkState, byte e1LinkState, byte e2LinkState, byte e3LinkState, byte e4LinkState,
                               byte e5LinkState, byte e6LinkState, byte e7LinkState, byte clockSource) {
        super(Globals.TYPE_RESPONSE, responseCode, Globals.OPERATION_SCAN, Globals.NONE, (short) 39);

        this.id = id;
        this.mac = mac;
        this.ip = ip;
        this.mask = mask;
        this.clockMode = clockMode;
        this.eE1LinkState = eE1LinkState;
        this.eE1LinkDeviceType = eE1LinkDeviceType;
        this.eE1LinkDevicePort = eE1LinkDevicePort;
        this.eE1LinkDeviceId = eE1LinkDeviceId;
        this.eE2LinkState = eE2LinkState;
        this.eE2LinkDeviceType = eE2LinkDeviceType;
        this.eE2LinkDevicePort = eE2LinkDevicePort;
        this.eE2LinkDeviceId = eE2LinkDeviceId;
        this.e0linkState = e0linkState;
        this.e1LinkState = e1LinkState;
        this.e2LinkState = e2LinkState;
        this.e3LinkState = e3LinkState;
        this.e4LinkState = e4LinkState;
        this.e5LinkState = e5LinkState;
        this.e6LinkState = e6LinkState;
        this.e7LinkState = e7LinkState;
        this.clockSource = clockSource;
    }

    @Override
    public void setPayload(ByteBuf buf) {
        buf.readByte();
        id = buf.readShort();
        mac = buf.readUnsignedInt() << 16 + buf.readUnsignedShort();
        ip = buf.readInt();
        mask = buf.readInt();
        buf.readByte();
        buf.readByte();
        clockMode = buf.readByte();
        eE1LinkState = buf.readByte();
        eE1LinkDeviceType = buf.readByte();
        eE1LinkDevicePort = buf.readByte();
        eE1LinkDeviceId = buf.readShort();
        eE2LinkState = buf.readByte();
        eE2LinkDeviceType = buf.readByte();
        eE2LinkDevicePort = buf.readByte();
        eE2LinkDeviceId = buf.readShort();
        e0linkState = buf.readByte();
        e1LinkState = buf.readByte();
        e2LinkState = buf.readByte();
        e3LinkState = buf.readByte();
        e4LinkState = buf.readByte();
        e5LinkState = buf.readByte();
        e6LinkState = buf.readByte();
        e7LinkState = buf.readByte();
        clockSource = buf.readByte();
    }

    @Override
    public ByteBuf getPayload() {
        ByteBuf buf = Unpooled.buffer(39);
        buf.writeByte(0);
        buf.writeShort(id);
        buf.writeInt((int) (mac >> 16));
        buf.writeShort((int) mac);
        buf.writeInt(ip);
        buf.writeInt(mask);
        buf.writeByte(0);
        buf.writeByte(0);
        buf.writeByte(clockMode);
        buf.writeByte(eE1LinkState);
        buf.writeByte(eE1LinkDeviceType);
        buf.writeByte(eE1LinkDevicePort);
        buf.writeShort(eE1LinkDeviceId);
        buf.writeByte(eE2LinkState);
        buf.writeByte(eE2LinkDeviceType);
        buf.writeByte(eE2LinkDevicePort);
        buf.writeShort(eE2LinkDeviceId);
        buf.writeByte(e0linkState);
        buf.writeByte(e1LinkState);
        buf.writeByte(e2LinkState);
        buf.writeByte(e3LinkState);
        buf.writeByte(e4LinkState);
        buf.writeByte(e5LinkState);
        buf.writeByte(e6LinkState);
        buf.writeByte(e7LinkState);
        buf.writeByte(clockSource);
        return buf;
    }

    @Override
    public String toString() {
        return "ScanResponsePackage{" +
                "id=" + id +
                ", mac=" + ConvertUtils.longMac2String(mac) +
                ", ip=" + ConvertUtils.ip2String(ip) +
                ", mask=" + ConvertUtils.ip2String(mask) +
                ", clockMode=" + Integer.toHexString(clockMode) +
                ", eE1LinkState=" + Integer.toHexString(eE1LinkState) +
                ", eE1LinkDeviceType=" + Integer.toHexString(eE1LinkDeviceType) +
                ", eE1LinkDevicePort=" + eE1LinkDevicePort +
                ", eE1LinkDeviceId=" + eE1LinkDeviceId +
                ", eE2LinkState=" + Integer.toHexString(eE2LinkState) +
                ", eE2LinkDeviceType=" + Integer.toHexString(eE2LinkDeviceType) +
                ", eE2LinkDevicePort=" + eE2LinkDevicePort +
                ", eE2LinkDeviceId=" + eE2LinkDeviceId +
                ", e0linkState=" + Integer.toHexString(e0linkState) +
                ", e1LinkState=" + Integer.toHexString(e1LinkState) +
                ", e2LinkState=" + Integer.toHexString(e2LinkState) +
                ", e3LinkState=" + Integer.toHexString(e3LinkState) +
                ", e4LinkState=" + Integer.toHexString(e4LinkState) +
                ", e5LinkState=" + Integer.toHexString(e5LinkState) +
                ", e6LinkState=" + Integer.toHexString(e6LinkState) +
                ", e7LinkState=" + Integer.toHexString(e7LinkState) +
                ", clockSource=" + Integer.toHexString(clockSource) +
                "} " + super.toString();
    }
}
