package db.entity.packages;

import db.Globals;
import db.entity.Strategy;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Data
@Slf4j
@ToString(callSuper = true)
public class StrategyWriteRequestPackage extends BasePackage {

    private int strategiesLen;
    private byte no;
    private List<Strategy> strategies;

    public StrategyWriteRequestPackage() {
        super(Globals.TYPE_REQUEST, Globals.NONE, Globals.OPERATION_WRITE, Globals.PACKAGE_STRATEGY, (short) 0);
    }

    @Override
    public void setPayload(ByteBuf buf) {
        strategiesLen = buf.readByte();
        no = buf.readByte();

        for (int i = 0; i < strategiesLen; i++) {
            Strategy strategy = new Strategy();
            strategy.setOperationType(buf.readByte());
            buf.readByte(); // byte1
            buf.readByte();
            buf.readByte();
            buf.readByte();
            buf.readByte(); // byte5
            strategy.setMac1(buf.readUnsignedInt() << 16 + buf.readUnsignedShort());
            buf.readByte();
            buf.readByte();
            strategy.setMac2(buf.readUnsignedInt() << 16 + buf.readUnsignedShort());
            strategy.setIp1Mask(buf.readInt());
            strategy.setIp2Mask(buf.readInt());
            strategy.setPort1Mask(buf.readShort());
            strategy.setPort2Mask(buf.readShort());
            strategies.add(strategy);
        }
    }

    @Override
    public ByteBuf getPayload() {
        int len = strategiesLen * 32 + 2;
        super.setPackageLength((short) len);
        ByteBuf buf = Unpooled.buffer(len);
        buf.writeByte(strategiesLen);
        buf.writeByte(no);
        for (int i = 0; i < strategiesLen; i++) {
            buf.writeByte(strategies.get(i).getOperationType());
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeInt((int) (strategies.get(i).getMac1() >> 16));
            buf.writeShort((int) strategies.get(i).getMac1());
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeInt((int) (strategies.get(i).getMac2() >> 16));
            buf.writeShort((int) strategies.get(i).getMac2());
            buf.writeInt(strategies.get(i).getIp1Mask());
            buf.writeInt(strategies.get(i).getIp2Mask());
            buf.writeShort(strategies.get(i).getPort1Mask());
            buf.writeShort(strategies.get(i).getPort2Mask());
        }
        return buf;
    }

}
