package db.entity.packages;

import db.Globals;
import db.entity.Table;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Data
@Slf4j
@ToString(callSuper = true)
public class TableInfoWriteRequestPackage extends BasePackage {

    private int tablesLen;
    private byte no;
    private List<Table> tables;

    public TableInfoWriteRequestPackage() {
        super(Globals.TYPE_REQUEST, Globals.NONE, Globals.OPERATION_WRITE, Globals.PACKAGE_TABLE, (short) 0);
    }

    @Override
    public void setPayload(ByteBuf buf) {
        tablesLen = buf.readByte();
        no = buf.readByte();

        for (int i = 0; i < tablesLen; i++) {
            Table table = new Table();
            table.setOperationType(buf.readByte());
            buf.readByte();
            table.setStrategyType(buf.readByte());
            table.setIsListen(buf.readByte());
            buf.readByte();
            buf.readByte();
            table.setMac1(buf.readUnsignedInt() << 16 + buf.readUnsignedShort());
            buf.readByte();
            buf.readByte();
            table.setMac2(buf.readUnsignedInt() << 16 + buf.readUnsignedShort());
            table.setIp1(buf.readInt());
            table.setIp2(buf.readInt());
            table.setPort1(buf.readShort());
            table.setPort2(buf.readShort());
            tables.add(table);
        }
    }

    @Override
    public ByteBuf getPayload() {
        int len = tablesLen * 32 + 2;
        super.setPackageLength((short) len);
        ByteBuf buf = Unpooled.buffer(len);
        buf.writeByte(tablesLen);
        buf.writeByte(no);
        for (int i = 0; i < tablesLen; i++) {
            buf.writeByte(tables.get(i).getOperationType());
            buf.writeByte(0);
            buf.writeByte(tables.get(i).getStrategyType());
            buf.writeByte(tables.get(i).getIsListen());
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeInt((int) (tables.get(i).getMac1() >> 16));
            buf.writeShort((int) tables.get(i).getMac1());
            buf.writeByte(0);
            buf.writeByte(0);
            buf.writeInt((int) (tables.get(i).getMac2() >> 16));
            buf.writeShort((int) tables.get(i).getMac2());
            buf.writeInt(tables.get(i).getIp1());
            buf.writeInt(tables.get(i).getIp2());
            buf.writeShort(tables.get(i).getPort1());
            buf.writeShort(tables.get(i).getPort2());
        }
        return buf;
    }

}
