package db.netty;

import db.entity.packages.BasePackage;
import db.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.codec.MessageToMessageEncoder;
import lombok.extern.slf4j.Slf4j;

import java.net.InetSocketAddress;
import java.util.List;


// 在这里编码包
@Slf4j
public class Encoder extends MessageToMessageEncoder<BasePackage> {

   @Override
   protected void encode(ChannelHandlerContext channelHandlerContext, BasePackage basePackage, List<Object> out) {

      DatagramPacket datagramPacket = encapsulate(basePackage);

      out.add(datagramPacket);

   }

   /**
    * 将报文封装成数据报
    * @param basePackage 待发送的报文
    * @return 封装后的数据报
    */
   private DatagramPacket encapsulate(BasePackage basePackage) {
      //写入并传送数据

      ByteBuf header = basePackage.getHeader();
      ByteBuf payload = basePackage.getPayload();

      ByteBuf buf = Unpooled.buffer(header.readableBytes()+ payload.readableBytes());

      buf.writeBytes(header);
      buf.writeBytes(payload);

      byte[] bytesReady = new byte[buf.readableBytes()];
      buf.getBytes(0, bytesReady);

//      log.debug("封装器将{}封装为{}", basePackage, ConvertUtils.bytes2HexArray(bytesReady));

      InetSocketAddress address = new InetSocketAddress(basePackage.getToHost(), basePackage.getToPort());

      return new DatagramPacket(buf, address);
   }
}