package db.controller;

import db.service.QueryService;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Map;

@RestController
@Slf4j
public class QueryController {

    @Autowired
    private QueryService queryService;

    @RequestMapping("/query")
    public String query(@RequestParam String start, @RequestParam String end,
                        @RequestParam Map<String, String> map) {
        try {

            map.remove("start");
            map.remove("end");

            String queryRes;
            if (map.keySet().size() == 0) {
                queryRes = queryService.queryByTime(start, end);
            } else {
                queryRes = queryService.queryByTimeAndCondition(start, end, map);
            }

            return queryRes;
        } catch (Exception e) {
            log.error("query出现错误", e);
            return "出错";
        }
    }

    @RequestMapping(value = "/download", method = RequestMethod.GET)
    public void getDownload(@RequestParam String pcapName, HttpServletRequest request, HttpServletResponse response) throws IOException {

        String fullPath = "./pcap/" + pcapName + ".pcap";
        File downloadFile = new File(fullPath);
        if (downloadFile.exists()) {

            String mimeType = "application/octet-stream";

            response.setContentType(mimeType);
            response.setContentLength((int) downloadFile.length());

            String headerKey = "Content-Disposition";
            String headerValue = String.format("attachment; filename=\"%s\"",
                    downloadFile.getName());
            response.setHeader(headerKey, headerValue);

            try (InputStream myStream = new FileInputStream(fullPath)) {
                IOUtils.copy(myStream, response.getOutputStream());
                response.flushBuffer();
                log.info("下载成功：{}", pcapName);
            } catch (IOException e) {
                log.info("下载失败：{}", pcapName);
                e.printStackTrace();
            }
        } else {
            response.setContentType("text/html;charset=utf-8");
            PrintWriter writer = response.getWriter();
            writer.write("未找到：" + pcapName);
            log.info("未找到：" + pcapName);
        }
    }

}
