#!/bin/sh

ROOT=$(cd `dirname $0`/../../; pwd)
cd $ROOT
APP_NAME=blackbox-insert
JAVA_HOME=$ROOT/jdk8
JAR_NAME=$ROOT/lib/insert-app-0.0.1-SNAPSHOT.jar
#PID  代表是PID文件
PID=$ROOT/pid/$APP_NAME\.pid


usage() {
    echo "Usage: sh 执行脚本.sh [start|stop|restart|status|force_stop]"
    exit 1
}

#检查程序是否在运行
is_exist(){
  pid=`ps -ef|grep $JAR_NAME|grep -v grep|awk '{print $2}' `
  #如果不存在返回1，存在返回0
  if [ -z "${pid}" ]; then
   return 1
  else
    return 0
  fi
}

#启动方法，--spring.profiles.active=prod0设置启动prod0环境
start(){
  is_exist
  if [ $? -eq "0" ]; then
    echo ">>> ${JAR_NAME} is already running PID=${pid} <<<"
  else
	  #nohup $JAVA_HOME/bin/java -Xms64m -Xmx1024m -Xss512K -Dspring.config.additional-location=conf/application-insert.yml -jar $JAR_NAME > /dev/null 2>&1 &
	  nohup $JAVA_HOME/bin/java -Xms64m -Xmx1024m -Xss512K -Dspring.config.additional-location=conf/application-insert.yml -jar $JAR_NAME >> ./logs/insert.log 2>&1 &
	  #java -Xms64m -Xmx1024m -Xss512K -Dspring.config.additional-location=conf/application-insert.yml -jar $JAR_NAME
    echo $! > $PID
    echo ">>> start $JAR_NAME successed PID=$! <<<"
  fi
}

#停止方法
stop(){
  #is_exist
  pidf=$(cat $PID)
  #echo "$pidf"
  echo ">>> APP PID = $pidf begin kill $pidf <<<"
  kill $pidf
  rm -rf $PID
  sleep 3
  is_exist
  if [ $? -eq "0" ]; then
    echo ">>> api 2 PID = $pid begin kill -9 $pid  <<<"
    kill -9  $pid
    sleep 2
    echo ">>> $JAR_NAME process stopped <<<"
  else
    echo ">>> ${JAR_NAME} is not running <<<"
  fi
}

force_stop(){
  #is_exist
  pidf=$(cat $PID)
  #echo "$pidf"
  echo ">>> APP PID = $pidf begin kill $pidf <<<"
  kill -9 $pidf
  rm -rf $PID
  sleep 3
  is_exist
  if [ $? -eq "0" ]; then
    echo ">>> api 2 PID = $pid begin kill -9 $pid  <<<"
    kill -9  $pid
    sleep 2
    echo ">>> $JAR_NAME process stopped <<<"
  else
    echo ">>> ${JAR_NAME} is not running <<<"
  fi
}

#输出运行状态
status(){
  is_exist
  if [ $? -eq "0" ]; then
    echo ">>> ${JAR_NAME} is running PID is ${pid} <<<"
  else
    echo ">>> ${JAR_NAME} is not running <<<"
  fi
}

#重启
restart(){
  stop
  start
}

#根据输入参数，选择执行对应方法，不输入则执行使用说明
case "$1" in
  "start")
    start
    ;;
  "stop")
    stop
    ;;
  "status")
    status
    ;;
  "restart")
    restart
    ;;
  "force_stop")
    restart
    ;;
  *)
    usage
    ;;
esac
exit 0
